/*
 * Copyright: STEM Loyola
 * Date     : August 2019
 * ID       : 19.08-B2
 * Level    : 2 (Intermediate)
 *
 * Task     : In 2014, NECTA released the form four (CSEE) results in GPA and
 *            class format (Figure 2). This came after they had introduced "B+"
 *            and "E" grades during the 2013 results. Your task is to convert
 *            the results to the points and divisions format (Figure 3) that we
 *            are all familiar with and count students in each division. A text
 *            file "2014-CSEE-S0800.txt" contains the results in GPA format as
 *            shown in Figure 1. No one got division zero but we have added one
 *            division-zero results to ensure you write a complete program.
 *
 *      Note: (1) Points and divisions are calculated from the 7 best grades as
 *                shown in Figure 4 and Figure 5.
 *            (2) In order to get Division IV, a student must have at least
 *                2 Ds or 1 C. This means, for example, that 6 Es and 1 D
 *                (i.e. 41 pts) will still be Division 0.
 *
 * Solved By: STEM Loyola Team
 *     Email: stemloyola@gmail.com
 *     Form : N/A
 *    Stream: N/A
*/

#include<iostream>  // std, cout, endl
#include<fstream>   // ifstream
#include<cstdlib>   // exit()
#include<sstream>   // stringstream
#include<algorithm> // sort()
#include<vector>
#include<map>

#define FILE_NAME    "2014-CSEE-S0800.txt"
#define MIN_SUBJECTS 7

using namespace std;

int main() {
    // Open the wards file
    ifstream resultsFile(FILE_NAME);

    // Ensure the file was opened successfully
    if ( resultsFile.is_open() == false ) {
        cout << "Error: could not open the file \"" << FILE_NAME << "\""
             << endl << "Quitting..." << endl;

        exit(1);    // Terminate the program
    }

    int totalDivisions[] = {0, 0, 0, 0, 0};  // Tracks students in each division

     // Build grade to points mapping
    map<string, int> gradeToPoints;
    gradeToPoints["A"]  = 1;
    gradeToPoints["B+"] = 2;
    gradeToPoints["B"]  = 3;
    gradeToPoints["C"]  = 4;
    gradeToPoints["D"]  = 5;
    gradeToPoints["E"]  = 6;
    gradeToPoints["F"]  = 7;

    // Parse the results files and extract students data
    string line;

    while ( getline(resultsFile, line) ) {
        stringstream ss(line);

        // Extract student ID and gender
        string id, gender;
        ss >> id >> gender;

        // Extract all subjects and grades for the current student
        vector<int> pointsList;

        string item;
        bool isGrade = false;

        while ( getline(ss, item, '\'') ) {
            if ( isGrade ) pointsList.push_back(gradeToPoints[item]);  // Collect grade (A, B+, etc.) as points (1, 2, etc.)

            isGrade = !isGrade;
        }

        // Compute and sum up the student's best seven points
        sort(pointsList.begin(), pointsList.end());

        int points = 0;
        for ( int i = 0; i < MIN_SUBJECTS; i++ ) {
            points += pointsList[i];
        }

        // Compute the student's division.
        string division = "";

        if ( points <= 17 ) {
            division = "I";
            totalDivisions[1]++;
        } else if ( points <= 24 ) {
            division = "II";
            totalDivisions[2]++;
        } else if ( points <= 31 ) {
            division = "III";
            totalDivisions[3]++;
        } else if ( points <= 46 ) {
            // At least one C or two D's are needed to get division 4 even
            // if a student scores the right points
            if ( pointsList[0] <= 4 || pointsList[0] + pointsList[1] <= 10 ) {
                division = "IV";
                totalDivisions[4]++;
            } else {
                division = "0";
                totalDivisions[0]++;
            }
        } else {
            division = "0";
            totalDivisions[0]++;
        }

        // Display the student's results
        cout << id << "\t" << gender << "\t" << points << "\t" << division << endl;
    }

    // Display the overall results summary
    cout << "\nResults summary:"  << endl;
    cout << "\tDivision I:\t"     << totalDivisions[1] << endl;
    cout << "\tDivision II:\t"    << totalDivisions[2] << endl;
    cout << "\tDivision III:\t"   << totalDivisions[3] << endl;
    cout << "\tDivision IV:\t"    << totalDivisions[4] << endl;
    cout << "\tDivision 0 (*):\t" << totalDivisions[0] << endl;

    // Close open resources
    resultsFile.close();

    return 0;
}
